package com.tencent.liteav.demo.superplayer.player;

import android.content.Context;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;

import com.tencent.liteav.demo.superplayer.SuperPlayerCode;
import com.tencent.liteav.demo.superplayer.SuperPlayerDef;
import com.tencent.liteav.demo.superplayer.SuperPlayerGlobalConfig;
import com.tencent.liteav.demo.superplayer.SuperPlayerModel;
import com.tencent.liteav.demo.superplayer.util.HistoryUtil;
import com.tencent.liteav.demo.superplayer.util.PlayerUtil;
import com.tencent.rtmp.ITXVodPlayListener;
import com.tencent.rtmp.TXLiveConstants;
import com.tencent.rtmp.TXVodPlayConfig;
import com.tencent.rtmp.TXVodPlayer;
import com.tencent.rtmp.ui.TXCloudVideoView;

import java.io.File;

public class SuperPlayerImpl implements SuperPlayer, ITXVodPlayListener {

    private static final String TAG = "SuperPlayerImpl";
    private static final int SUPERPLAYER_MODE = 1;
    private static final int SUPPORT_MAJOR_VERSION = 8;
    private static final int SUPPORT_MINOR_VERSION = 5;

    private Context mContext;
    private TXCloudVideoView mVideoView;        // 腾讯云视频播放view

    private TXVodPlayer mVodPlayer;       // 点播播放器
    private TXVodPlayConfig mVodPlayConfig;   // 点播播放器配置

    private SuperPlayerModel mCurrentModel;  // 当前播放的model
    private SuperPlayerObserver mObserver;
    private SuperPlayerDef.PlayerMode mCurrentPlayMode = SuperPlayerDef.PlayerMode.WINDOW;    // 当前播放模式
    private SuperPlayerDef.PlayerState mCurrentPlayState = SuperPlayerDef.PlayerState.INIT;

    private String mCurrentPlayVideoURL;    // 当前播放的URL
    private float mStartPos;                // 视频开始播放时间

    private boolean isPrepared = false;
    private boolean isNeedResume = false;
    private boolean mNeedToPause = false;
    private boolean mTimeout = false;
    private boolean isPauseByTimeout = false;
    private boolean isStartBySeek = false;

    private long mCurrent = 0L;
    private long mStartPoint = 0L;
    private long mEndPoint = 0L;

    public SuperPlayerImpl(Context context, TXCloudVideoView videoView) {
        mContext = context;
        mVideoView = videoView;

        initVodPlayer(mContext);
    }

    @Override
    public void setNeedToPause(boolean value) {
        mNeedToPause = value;
    }


    /**
     * 点播播放器事件回调
     *
     * @param player
     * @param event
     * @param param
     */
    @Override
    public void onPlayEvent(TXVodPlayer player, int event, Bundle param) {
        if (event != TXLiveConstants.PLAY_EVT_PLAY_PROGRESS) {
            String playEventLog = "TXVodPlayer onPlayEvent event: " + event + ", " + param.getString(TXLiveConstants.EVT_DESCRIPTION);
            Log.d(TAG, playEventLog);
        }
        switch (event) {
            case TXLiveConstants.PLAY_EVT_VOD_PLAY_PREPARED://视频播放开始
                onVodPlayPrepared();
                break;
            case TXLiveConstants.PLAY_EVT_RCV_FIRST_I_FRAME:
                Log.i(TAG, "PLAY_EVT_RCV_FIRST_I_FRAME");
                if (mNeedToPause) {
                    return;
                }
                updatePlayerState(SuperPlayerDef.PlayerState.PLAYING);
                mObserver.onRcvFirstIframe();
                break;
            case TXLiveConstants.PLAY_EVT_PLAY_END:
                Log.i(TAG, "PLAY_EVT_PLAY_END");
                updatePlayerState(SuperPlayerDef.PlayerState.END);
                break;
            case TXLiveConstants.PLAY_EVT_PLAY_PROGRESS:
                int progress = param.getInt(TXLiveConstants.EVT_PLAY_PROGRESS_MS);
                int duration = param.getInt(TXLiveConstants.EVT_PLAY_DURATION_MS);
                mCurrent = progress / 1000;

                if (duration != 0) {
                    updatePlayProgress(progress / 1000, duration / 1000);
                }
                break;
            case TXLiveConstants.PLAY_EVT_PLAY_LOADING:
                Log.i(TAG, "PLAY_EVT_PLAY_LOADING");
                updatePlayerState(SuperPlayerDef.PlayerState.LOADING);
                break;
            case TXLiveConstants.PLAY_EVT_PLAY_BEGIN:
                if (mNeedToPause) {
                    pause();
                    return;
                }
                updatePlayerState(SuperPlayerDef.PlayerState.PLAYING);
                break;
            default:
                break;
        }
        if (event < 0) {// 播放点播文件失败
            mVodPlayer.stopPlay(true);
            updatePlayerState(SuperPlayerDef.PlayerState.PAUSE);
            onError(SuperPlayerCode.VOD_PLAY_FAIL, param.getString(TXLiveConstants.EVT_DESCRIPTION));
        }
    }

    private void onVodPlayPrepared() {
        isPrepared = true;

        if (mNeedToPause) {
            pause();
            return;
        }
        if (isNeedResume) {
            mVodPlayer.resume();
        }
    }

    /**
     * 点播播放器网络状态回调
     *
     * @param player
     * @param bundle
     */
    @Override
    public void onNetStatus(TXVodPlayer player, Bundle bundle) {
        // DO NOTHING
    }

    /**
     * 初始化点播播放器
     *
     * @param context
     */
    private void initVodPlayer(Context context) {
        mVodPlayer = new TXVodPlayer(context);
        SuperPlayerGlobalConfig config = SuperPlayerGlobalConfig.getInstance();
        mVodPlayConfig = new TXVodPlayConfig();

        File sdcardDir = context.getExternalFilesDir(null);
        if (sdcardDir != null) {
            mVodPlayConfig.setCacheFolderPath(sdcardDir.getPath() + "/txcache");
        }
        mVodPlayConfig.setPreferredResolution(720 * 1280);
        mVodPlayConfig.setMaxCacheItems(config.maxCacheItem);
        mVodPlayConfig.setHeaders(config.headers);
        mVodPlayer.setConfig(mVodPlayConfig);
        mVodPlayer.setRenderMode(config.renderMode);
        mVodPlayer.setVodListener(this);
        mVodPlayer.enableHardwareDecode(config.enableHWAcceleration);
        mVodPlayer.setRate(config.playRate);
        mVodPlayer.setMute(config.mute);
        mVodPlayer.setMirror(config.mirror);
    }

    /**
     * 播放视频
     *
     * @param model
     */
    public void playWithModel(SuperPlayerModel model) {
        reset();

        String videoURL = model.url;

        if (TextUtils.isEmpty(videoURL)) {
            onError(SuperPlayerCode.PLAY_URL_EMPTY, "播放视频失败，播放链接为空");
            return;
        }

        mVodPlayer.setPlayerView(mVideoView);
        playVodURL(videoURL);

        updatePlayProgress(0, model.duration);
    }

    /**
     * 播放点播url
     */
    private void playVodURL(String url) {
        if (url == null || "".equals(url)) {
            return;
        }
        mCurrentPlayVideoURL = url;

        if (mVodPlayer != null) {
            mVodPlayer.setStartTime(mStartPos);
            mVodPlayer.setAutoPlay(true);
            mVodPlayer.setVodListener(this);
            mVodPlayer.setToken(null);
            mVodPlayer.startPlay(url);
        }
    }

    /**
     * 更新播放进度
     *
     * @param current  当前播放进度(秒)
     * @param duration 总时长(秒)
     */
    private void updatePlayProgress(long current, long duration) {
        if (mObserver != null) {
            mObserver.onPlayProgress(current, duration);
        }
    }

    /**
     * 更新播放状态
     *
     * @param playState
     */
    private void updatePlayerState(SuperPlayerDef.PlayerState playState) {
        mCurrentPlayState = playState;
        if (mObserver == null) {
            return;
        }
        switch (playState) {
            case INIT:
                mObserver.onPlayPrepare();
                break;
            case PLAYING:
                PlayerUtil.INSTANCE.sendNewTimer(PlayerUtil.getSTATE_PLAY());
                mObserver.onPlayBegin(getPlayName());
                break;
            case PAUSE:
                PlayerUtil.INSTANCE.sendNewTimer(PlayerUtil.getSTATE_PAUSE());
                mObserver.onPlayPause();
                break;
            case LOADING:
                PlayerUtil.INSTANCE.sendNewTimer(PlayerUtil.getSTATE_LOADING());
                mObserver.onPlayLoading();
                break;
            case END:
                PlayerUtil.INSTANCE.sendNewTimer(PlayerUtil.getSTATE_STOP());
                mObserver.onPlayStop();
                break;
        }

        switch (playState) {
            case PLAYING:
                if (isStartBySeek) {
                    isStartBySeek = false;
                } else {
                    mStartPoint = mCurrent;
                }
                break;
            case PAUSE:
            case END:
                mEndPoint = mCurrent;
                if (mCurrentModel != null) {
                    HistoryUtil.INSTANCE.sendHistory(mStartPoint, mEndPoint, mCurrentModel.sectionId);
                }
                break;
        }
    }

    private void onError(int code, String message) {
        if (mObserver != null) {
            mObserver.onError(code, message);
        }
    }

    private String getPlayName() {
        String title = "";
        if (mCurrentModel != null && !TextUtils.isEmpty(mCurrentModel.title)) {
            title = mCurrentModel.title;
        }
        return title;
    }

    @Override
    public void play(SuperPlayerModel model) {
        if (mTimeout) return;

        mCurrentModel = model;
        playWithModel(model);
    }

    @Override
    public void restart() {
        if (mTimeout) return;

        if (mCurrentPlayVideoURL != null) {
            playVodURL(mCurrentPlayVideoURL);
        } else {
            playWithModel(mCurrentModel);
        }
    }

    @Override
    public void pause() {
        mVodPlayer.pause();
        updatePlayerState(SuperPlayerDef.PlayerState.PAUSE);
    }

    @Override
    public void resume() {
        if (mTimeout) return;

        isNeedResume = true;
        if (isPrepared) {
            mVodPlayer.resume();
        }

        updatePlayerState(SuperPlayerDef.PlayerState.PLAYING);
    }

    @Override
    public void stop() {
        resetPlayer();
        updatePlayerState(SuperPlayerDef.PlayerState.END);
    }

    @Override
    public void reset() {
        resetPlayer();
        updatePlayerState(SuperPlayerDef.PlayerState.INIT);
    }

    @Override
    public void onTimeout(boolean timeout) {
        this.mTimeout = timeout;

        if (timeout) {
            if (mCurrentPlayState == SuperPlayerDef.PlayerState.PLAYING || mCurrentPlayState == SuperPlayerDef.PlayerState.LOADING) {
                isPauseByTimeout = true;
                pause();
            }
        } else {
            mStartPoint = mEndPoint;

            if (mCurrentModel != null) {
                HistoryUtil.INSTANCE.sendHistory(mStartPoint, mEndPoint, mCurrentModel.sectionId);
            }

            if (isPauseByTimeout) {
                resume();
            }

        }
    }

    private void resetPlayer() {
        isPrepared = false;
        isNeedResume = false;
        if (mVodPlayer != null) {
            mVodPlayer.setVodListener(null);
            mVodPlayer.stopPlay(false);
        }
    }

    @Override
    public void switchPlayMode(SuperPlayerDef.PlayerMode playerMode) {
        if (mCurrentPlayMode == playerMode) {
            return;
        }
        mCurrentPlayMode = playerMode;

        if (mCurrentPlayMode == SuperPlayerDef.PlayerMode.FULLSCREEN) {
            if (getPlayerState() == SuperPlayerDef.PlayerState.PAUSE) {
                resume();
            }
        }
    }

    @Override
    public void seek(int position) {
        if(mCurrentModel != null) {
            HistoryUtil.INSTANCE.sendHistory(mStartPoint, mEndPoint, mCurrentModel.sectionId);
        }

        mStartPoint = position;
        isStartBySeek = true;

        if (mVodPlayer != null) {
            mVodPlayer.seek(position);
            if (!mVodPlayer.isPlaying()) {
                mVodPlayer.resume();
            }
        }

        if (mObserver != null) {
            mObserver.onSeek(position);
        }
    }

    @Override
    public SuperPlayerDef.PlayerMode getPlayerMode() {
        return mCurrentPlayMode;
    }

    @Override
    public SuperPlayerDef.PlayerState getPlayerState() {
        return mCurrentPlayState;
    }

    @Override
    public void setObserver(SuperPlayerObserver observer) {
        mObserver = observer;
    }

    @Override
    public void setLoop(boolean isLoop) {
        mVodPlayer.setLoop(isLoop);
    }

    @Override
    public void setStartTime(float startPos) {
        this.mStartPos = startPos;
        mVodPlayer.setStartTime(startPos);
    }
}
