package com.tencent.liteav.demo.player

import android.content.Intent
import android.os.Bundle
import android.util.Log
import android.view.View
import android.view.WindowManager
import android.widget.Toast
import androidx.activity.viewModels
import androidx.appcompat.app.AppCompatActivity
import androidx.recyclerview.widget.LinearLayoutManager
import com.tencent.liteav.demo.player.ViewInsets.ViewportMetrics
import com.tencent.liteav.demo.player.databinding.ActivityPlayerBinding
import com.tencent.liteav.demo.player.ui.PlayerListAdapter
import com.tencent.liteav.demo.player.ui.PlayerListDecoration
import com.tencent.liteav.demo.player.util.PlayerModelProvider
import com.tencent.liteav.demo.player.viewmodel.PlayerViewModel
import com.tencent.liteav.demo.player.viewmodel.PlayerViewModelFactory
import com.tencent.liteav.demo.superplayer.SuperPlayerDef
import com.tencent.liteav.demo.superplayer.SuperPlayerGlobalConfig
import com.tencent.liteav.demo.superplayer.SuperPlayerModel
import com.tencent.liteav.demo.superplayer.SuperPlayerView.OnSuperPlayerViewCallback
import com.tencent.liteav.demo.superplayer.database.PlayerDatabaseProvider
import com.tencent.liteav.demo.superplayer.util.TimeoutUtil
import com.tencent.liteav.demo.superplayer.util.TimersUtil
import com.tencent.rtmp.TXLiveConstants
import kotlinx.coroutines.CoroutineScope
import kotlinx.coroutines.MainScope
import kotlinx.coroutines.launch

/**
 * Created by liyuejiao on 2018/7/3.
 * 超级播放器主Activity
 */
class PlayerActivity : AppCompatActivity(),
    View.OnClickListener,
    OnSuperPlayerViewCallback,
    PlayerListAdapter.OnItemClickListener,
    CoroutineScope by MainScope(), TimeoutUtil.Listener {

    private var _viewBinding: ActivityPlayerBinding? = null
    private val viewBinding get() = _viewBinding!!

    private val activityLayout get() = viewBinding.superplayerActivity
    private val mLayoutTopBar get() = viewBinding.superplayerTcActionBar
    private val mLayoutList get() = viewBinding.superplayerRbList
    private val mLayoutPlayer get() = viewBinding.superplayerBlPlayer
    private val mImageBack get() = viewBinding.superplayerTcBack
    private val mTextTitle get() = viewBinding.superplayerTcUrl
    private val mImageLock get() = viewBinding.superplayerTcLock
    private val mImageMenu get() = viewBinding.superplayerTcMenu
    private val mSuperPlayerView get() = viewBinding.superVodPlayerView
    private val mVodPlayerListView get() = viewBinding.superplayerRecyclerView
    private lateinit var mVodPlayerListAdapter: PlayerListAdapter

    private var mIsManualPause = false

    private var viewInsets = ViewportMetrics()

    private val viewModel by viewModels<PlayerViewModel> {
        PlayerViewModelFactory(
            PlayerModelProvider.getCourseId(),
            ((application) as PlayerDatabaseProvider).getPlayerRepository()
        )
    }

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        _viewBinding = ActivityPlayerBinding.inflate(layoutInflater)
        setContentView(viewBinding.root)
        window.addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON)
        initView()
        initData()
        updateList()
        viewModel.loadConfig(this)
        observe()
    }

    private fun observe() {
        viewModel.toastStr.observe(this) {
            Toast.makeText(this, it, Toast.LENGTH_SHORT).show()
        }
    }


    override fun onBackPressed() {
        if (isFullScreen) {
            mSuperPlayerView.switchPlayMode(SuperPlayerDef.PlayerMode.WINDOW)
        } else {
            super.onBackPressed()
        }
    }

    /**
     * 以16：9 比例显示播放器view，优先保证宽度完全填充
     */
    private fun adjustSuperPlayerViewAndMaskHeight() {
        val screenWidth = windowManager.defaultDisplay.width
        val layoutParams = mSuperPlayerView.layoutParams
        layoutParams.width = screenWidth
        layoutParams.height = (screenWidth * sPlayerViewDisplayRatio).toInt()
        mSuperPlayerView.layoutParams = layoutParams
        window.decorView.setOnApplyWindowInsetsListener { v, insets ->
            viewInsets = ViewInsets.applyWindowInsets(this@PlayerActivity, insets)
            setPadding()
            insets
        }
    }

    private fun initView() {
        mImageBack.setOnClickListener(this)
        mTextTitle.setOnClickListener(this)
        mImageLock.setOnClickListener(this)
        mImageMenu.setOnClickListener(this)
        mSuperPlayerView.setPlayerViewCallback(this)

        mVodPlayerListView.layoutManager = LinearLayoutManager(this).apply {
            isSmoothScrollbarEnabled = true
        }
        mVodPlayerListAdapter = PlayerListAdapter(this)
        mVodPlayerListAdapter.setOnItemClickListener(this)
        mVodPlayerListView.adapter = mVodPlayerListAdapter
        mVodPlayerListView.addItemDecoration(PlayerListDecoration())

        adjustSuperPlayerViewAndMaskHeight()
    }

    /**
     * 初始化超级播放器全局配置
     */
    private fun initData() {
        val prefs = SuperPlayerGlobalConfig.getInstance()
        prefs.maxCacheItem = 5
        prefs.enableHWAcceleration = true
        prefs.renderMode = TXLiveConstants.RENDER_MODE_ADJUST_RESOLUTION
    }

    override fun onResume() {
        super.onResume()
        TimeoutUtil.addListener(this)
        if (mSuperPlayerView.playerState == SuperPlayerDef.PlayerState.PLAYING
            || mSuperPlayerView.playerState == SuperPlayerDef.PlayerState.PAUSE
        ) {
            Log.i(TAG, "onResume state :" + mSuperPlayerView.playerState)
            if (!mIsManualPause) {
                mSuperPlayerView.onResume()
            }
        }
        mSuperPlayerView.setNeedToPause(false)
    }

    override fun onPause() {
        super.onPause()
        TimeoutUtil.removeListener(this)
        Log.i(TAG, "onPause state :" + mSuperPlayerView.playerState)
        mIsManualPause = mSuperPlayerView.playerState == SuperPlayerDef.PlayerState.PAUSE
        mSuperPlayerView.onPause()
        mSuperPlayerView.setNeedToPause(true)
    }

    override fun onStop() {
        super.onStop()
        viewModel.onStop()
    }

    override fun onDestroy() {
        super.onDestroy()

        mSuperPlayerView.release()
        mSuperPlayerView.resetPlayer()
        _viewBinding = null
    }

    override fun onItemClick(videoModel: SuperPlayerModel, index: Int) {
        playVideoModel(index)
    }

    override fun onTimeout(timeout: Boolean) {
        if (!timeout) {
            if (mSuperPlayerView.playingVideoModel == null) {
                playVideoModel(10)
                mSuperPlayerView.setStartTime(20.0)
            }
        }
    }

    private fun playVideoModel(index: Int) {
        mSuperPlayerView.maybePlayIndexModel(index)
    }

    override fun onClick(v: View) {
        when (v.id) {
            R.id.superplayer_tc_back -> {
                finish()
            }
            R.id.superplayer_tc_lock -> {
                mSuperPlayerView.switchPlayMode(SuperPlayerDef.PlayerMode.FULLSCREEN)
            }
            R.id.superplayer_tc_menu -> {
                PlayerMenu.start(this@PlayerActivity, "")
            }
        }
    }

    private fun updateList() {
        mVodPlayerListAdapter.clear()

        val list = PlayerModelProvider.getList();
        for (videoModel in list) {
            mVodPlayerListAdapter.addSuperPlayerModel(videoModel)
        }

        mSuperPlayerView.setupModelList(list)
    }

    private var isFullScreen = false
    override fun onStartFullScreenPlay() {
        // 隐藏其他元素实现全屏
        mLayoutPlayer.onFullScreenMode()
        mLayoutTopBar.visibility = View.GONE
        mLayoutList.visibility = View.GONE
        isFullScreen = true
        setPadding()
    }

    override fun onStopFullScreenPlay() {
        // 恢复原有元素
        mLayoutPlayer.onWindowMode()
        mLayoutTopBar.visibility = View.VISIBLE
        mLayoutList.visibility = View.VISIBLE
        isFullScreen = false
        setPadding()
    }

    private fun setPadding() {
        if (!isFullScreen) {
            activityLayout.setPadding(
                viewInsets.viewPaddingLeft,
                viewInsets.viewPaddingTop,
                viewInsets.viewPaddingRight,
                viewInsets.viewPaddingBottom
            )
        } else {
            activityLayout.setPadding(0, 0, 0, 0)
        }
    }

    override fun onPlayPrepare(index: Int, model: SuperPlayerModel) {
        if (viewModel.checkCanPlay()) {
            mSuperPlayerView.playIndexModel(index)
        } else {
            Toast.makeText(this, "定时时间到了~", Toast.LENGTH_SHORT).show()
        }
    }

    override fun onPlayIndex(index: Int, model: SuperPlayerModel) {
        launch {
            mVodPlayerListAdapter.setIndex(index)
            mVodPlayerListView.smoothScrollToPosition(index)
        }
    }

    override fun onPlayEnd() {}

    override fun onPlaying() {}

    override fun onVerify(reason: SuperPlayerDef.VerifyReason) {
        if (reason == SuperPlayerDef.VerifyReason.TIMEOUT) {
            PlayerVerify.start(this@PlayerActivity, 0)
        }
    }

    override fun onActivityResult(requestCode: Int, resultCode: Int, data: Intent?) {
        super.onActivityResult(requestCode, resultCode, data)
        if (requestCode == 0) {
            if (resultCode == RESULT_OK) {
                TimersUtil.sendNewTimer(0, 0)
            }
        }
    }

    override fun onResumePlay() {
//        viewModel.resumeTimer()
    }

    override fun onPausePlay() {
//        viewModel.pauseTimer()
    }

    override fun onError(code: Int) {}

    companion object {
        private const val TAG = "SuperPlayerActivity"
        private const val sPlayerViewDisplayRatio =
            720.toFloat() / 1280 //当前界面播放器view展示的宽高比，用主流的16：9
    }
}